@@ -51,6 +51,8 @@ foreach_crtc (GfMonitor          *monitor,
   ForeachCrtcData *data = user_data;
 
   data->func (data->logical_monitor,
+              monitor,
+              monitor_crtc_mode->output,
               gf_output_get_assigned_crtc (monitor_crtc_mode->output),
               data->user_data);
 
@@ -90,7 +92,7 @@ derive_monitor_transform (GfMonitor *monitor)
   GfMonitorTransform transform;
 
   main_output = gf_monitor_get_main_output (monitor);
-  transform = gf_output_get_assigned_crtc (main_output)->transform;
+  transform = gf_output_get_assigned_crtc (main_output)->config->transform;
 
   return gf_monitor_crtc_to_logical_transform (monitor, transform);
 }
@@ -213,18 +215,15 @@ gf_logical_monitor_add_monitor (GfLogicalMonitor *logical_monitor,
       for (l_output = outputs; l_output; l_output = l_output->next)
         {
           GfOutput *output;
-          GfCrtc *crtc;
 
           output = l_output->data;
           is_presentation = is_presentation && output->is_presentation;
-          crtc = gf_output_get_assigned_crtc (output);
-
-          if (crtc)
-            crtc->logical_monitor = logical_monitor;
         }
     }
 
   logical_monitor->is_presentation = is_presentation;
+
+  gf_monitor_set_logical_monitor (monitor, logical_monitor);
 }
 
 gboolean
