@@ -665,6 +665,7 @@ assign_monitor_crtc (GfMonitor          *monitor,
   GfCrtc *crtc;
   GfMonitorTransform transform;
   GfMonitorTransform crtc_transform;
+  GfMonitorTransform crtc_hw_transform;
   int crtc_x, crtc_y;
   float x_offset, y_offset;
   float scale;
@@ -693,9 +694,12 @@ assign_monitor_crtc (GfMonitor          *monitor,
 
   transform = data->logical_monitor_config->transform;
   crtc_transform = gf_monitor_logical_to_crtc_transform (monitor, transform);
-  if (!gf_monitor_manager_is_transform_handled (data->monitor_manager,
-                                                crtc, crtc_transform))
-    crtc_transform = GF_MONITOR_TRANSFORM_NORMAL;
+  if (gf_monitor_manager_is_transform_handled (data->monitor_manager,
+                                               crtc,
+                                               crtc_transform))
+    crtc_hw_transform = crtc_transform;
+  else
+    crtc_hw_transform = GF_MONITOR_TRANSFORM_NORMAL;
 
   gf_monitor_calculate_crtc_pos (monitor, mode, output, crtc_transform,
                                  &crtc_x, &crtc_y);
@@ -738,7 +742,7 @@ assign_monitor_crtc (GfMonitor          *monitor,
     .crtc = crtc,
     .mode = crtc_mode,
     .layout = crtc_layout,
-    .transform = crtc_transform,
+    .transform = crtc_hw_transform,
     .outputs = g_ptr_array_new ()
   };
   g_ptr_array_add (crtc_info->outputs, output);
@@ -862,23 +866,35 @@ GfMonitorsConfigKey *
 gf_create_monitors_config_key_for_current_state (GfMonitorManager *monitor_manager)
 {
   GfMonitorsConfigKey *config_key;
+  GfMonitorSpec *laptop_monitor_spec;
   GList *l;
   GList *monitor_specs;
 
+  laptop_monitor_spec = NULL;
   monitor_specs = NULL;
   for (l = monitor_manager->monitors; l; l = l->next)
     {
       GfMonitor *monitor = l->data;
       GfMonitorSpec *monitor_spec;
 
-      if (gf_monitor_is_laptop_panel (monitor) &&
-          gf_monitor_manager_is_lid_closed (monitor_manager))
-        continue;
+      if (gf_monitor_is_laptop_panel (monitor))
+        {
+          laptop_monitor_spec = gf_monitor_get_spec (monitor);
+
+          if (gf_monitor_manager_is_lid_closed (monitor_manager))
+            continue;
+        }
 
       monitor_spec = gf_monitor_spec_clone (gf_monitor_get_spec (monitor));
       monitor_specs = g_list_prepend (monitor_specs, monitor_spec);
     }
 
+  if (!monitor_specs && laptop_monitor_spec)
+    {
+      monitor_specs =
+        g_list_prepend (NULL, gf_monitor_spec_clone (laptop_monitor_spec));
+    }
+
   if (!monitor_specs)
     return NULL;
 
