@@ -53,16 +53,18 @@
 
 typedef struct
 {
-  GfGpu         *gpu;
+  GfGpu            *gpu;
 
-  GList         *outputs;
-  GList         *modes;
-  GHashTable    *mode_ids;
+  GList            *outputs;
+  GList            *modes;
+  GHashTable       *mode_ids;
 
-  GfMonitorMode *preferred_mode;
-  GfMonitorMode *current_mode;
+  GfMonitorMode    *preferred_mode;
+  GfMonitorMode    *current_mode;
 
-  GfMonitorSpec *spec;
+  GfMonitorSpec    *spec;
+
+  GfLogicalMonitor *logical_monitor;
 
   /*
    * The primary or first output for this monitor, 0 if we can't figure out.
@@ -73,9 +75,9 @@ typedef struct
    * (it's an attempt to keep windows on the same monitor, and preferably on
    * the primary one).
    */
-  glong          winsys_id;
+  glong             winsys_id;
 
-  char          *display_name;
+  char             *display_name;
 } GfMonitorPrivate;
 
 enum
@@ -206,7 +208,7 @@ is_current_mode_known (GfMonitor *monitor)
   output = gf_monitor_get_main_output (monitor);
   crtc = gf_output_get_assigned_crtc (output);
 
-  return gf_monitor_is_active (monitor) == (crtc && crtc->current_mode);
+  return gf_monitor_is_active (monitor) == (crtc && crtc->config);
 }
 
 static gboolean
@@ -550,7 +552,8 @@ gf_monitor_is_mode_assigned (GfMonitor     *monitor,
       crtc = gf_output_get_assigned_crtc (output);
 
       if (monitor_crtc_mode->crtc_mode &&
-          (!crtc || crtc->current_mode != monitor_crtc_mode->crtc_mode))
+          (!crtc || !crtc->config ||
+           crtc->config->mode != monitor_crtc_mode->crtc_mode))
         return FALSE;
       else if (!monitor_crtc_mode->crtc_mode && crtc)
         return FALSE;
@@ -881,16 +884,10 @@ gf_monitor_logical_to_crtc_transform (GfMonitor          *monitor,
                                       GfMonitorTransform  transform)
 {
   GfOutput *output;
-  GfMonitorTransform new_transform;
 
   output = gf_monitor_get_main_output (monitor);
-  new_transform = (transform + output->panel_orientation_transform) %
-                  GF_MONITOR_TRANSFORM_FLIPPED;
 
-  if (gf_monitor_transform_is_flipped (transform))
-    new_transform += GF_MONITOR_TRANSFORM_FLIPPED;
-
-  return new_transform;
+  return gf_output_logical_to_crtc_transform (output, transform);
 }
 
 GfMonitorTransform
@@ -898,17 +895,10 @@ gf_monitor_crtc_to_logical_transform (GfMonitor          *monitor,
                                       GfMonitorTransform  transform)
 {
   GfOutput *output;
-  GfMonitorTransform new_transform;
 
   output = gf_monitor_get_main_output (monitor);
-  new_transform = (transform + GF_MONITOR_TRANSFORM_FLIPPED -
-                   output->panel_orientation_transform) %
-                  GF_MONITOR_TRANSFORM_FLIPPED;
 
-  if (gf_monitor_transform_is_flipped (transform))
-    new_transform += GF_MONITOR_TRANSFORM_FLIPPED;
-
-  return new_transform;
+  return gf_output_crtc_to_logical_transform (output, transform);
 }
 
 gboolean
@@ -922,16 +912,11 @@ gf_monitor_get_suggested_position (GfMonitor *monitor,
 GfLogicalMonitor *
 gf_monitor_get_logical_monitor (GfMonitor *monitor)
 {
-  GfOutput *output;
-  GfCrtc *crtc;
+  GfMonitorPrivate *priv;
 
-  output = gf_monitor_get_main_output (monitor);
-  crtc = gf_output_get_assigned_crtc (output);
+  priv = gf_monitor_get_instance_private (monitor);
 
-  if (crtc)
-    return crtc->logical_monitor;
-  else
-    return NULL;
+  return priv->logical_monitor;
 }
 
 GfMonitorMode *
@@ -1254,3 +1239,14 @@ gf_monitor_has_aspect_as_size (GfMonitor *monitor)
          (width_mm == 16 && height_mm == 9) ||
          (width_mm == 16 && height_mm == 10);
 }
+
+void
+gf_monitor_set_logical_monitor (GfMonitor        *monitor,
+                                GfLogicalMonitor *logical_monitor)
+{
+  GfMonitorPrivate *priv;
+
+  priv = gf_monitor_get_instance_private (monitor);
+
+  priv->logical_monitor = logical_monitor;
+}
