@@ -73,7 +73,7 @@ get_average_color (cairo_surface_t *surface,
   red_total = 0;
   green_total = 0;
   blue_total = 0;
-  pixels_total = width * height;
+  pixels_total = (guint64) width * height;
 
   for (row = 0; row < height; row++)
     {
@@ -371,6 +371,8 @@ gf_background_surface_create (GdkDisplay *display,
 
   cr = cairo_create (surface);
 
+  color = (GdkRGBA) {};
+
   n_monitors = gdk_display_get_n_monitors (display);
 
   for (i = 0; i < n_monitors; i++)
