@@ -95,6 +95,8 @@ enum
 
   ACTIVATE,
 
+  RENAME,
+
   TRASH,
   DELETE,
 
@@ -624,6 +626,14 @@ create_icon_info (GfIconView *self,
   return gf_icon_info_new (icon);
 }
 
+static void
+icon_changed_cb (GfIcon     *icon,
+                 GfIconView *self)
+{
+  if (self->placement == GF_PLACEMENT_AUTO_ARRANGE_ICONS)
+    resort_icons (self, FALSE);
+}
+
 static void
 query_info_cb (GObject      *object,
                GAsyncResult *res,
@@ -655,6 +665,8 @@ query_info_cb (GObject      *object,
   icon = gf_icon_new (self, file, file_info);
   g_object_unref (file_info);
 
+  g_signal_connect (icon, "changed", G_CALLBACK (icon_changed_cb), self);
+
   icon_info = create_icon_info (self, icon);
   self->icons = g_list_prepend (self->icons, icon_info);
 
@@ -662,6 +674,8 @@ query_info_cb (GObject      *object,
     resort_icons (self, TRUE);
   else
     add_icons (self);
+
+  gf_icon_update (GF_ICON (icon));
 }
 
 static GfIconInfo *
@@ -709,6 +723,19 @@ remove_icon_from_view (GfIconView *self,
     self->extend_from_icon = NULL;
 }
 
+static void
+remove_icon (GfIconView *self,
+             GfIconInfo *info)
+{
+  remove_icon_from_view (self, info);
+
+  self->icons = g_list_remove (self->icons, info);
+  gf_icon_info_free (info);
+
+  if (self->placement == GF_PLACEMENT_AUTO_ARRANGE_ICONS)
+    remove_and_readd_icons (self);
+}
+
 static void
 file_deleted (GfIconView *self,
               GFile      *deleted_file)
@@ -720,13 +747,7 @@ file_deleted (GfIconView *self,
   if (info == NULL)
     return;
 
-  remove_icon_from_view (self, info);
-
-  self->icons = g_list_remove (self->icons, info);
-  gf_icon_info_free (info);
-
-  if (self->placement == GF_PLACEMENT_AUTO_ARRANGE_ICONS)
-    remove_and_readd_icons (self);
+  remove_icon (self, info);
 }
 
 static void
@@ -753,14 +774,25 @@ file_renamed (GfIconView *self,
               GFile      *old_file,
               GFile      *new_file)
 {
-  GfIconInfo *info;
+  GfIconInfo *old_info;
+  GfIconInfo *new_info;
 
-  info = find_icon_info_by_file (self, old_file);
+  old_info = find_icon_info_by_file (self, old_file);
+  new_info = find_icon_info_by_file (self, new_file);
 
-  if (info == NULL)
-    return;
-
-  gf_icon_set_file (GF_ICON (info->icon), new_file);
+  if (old_info != NULL && new_info != NULL)
+    {
+      gf_icon_set_file (GF_ICON (new_info->icon), new_file);
+      remove_icon (self, old_info);
+    }
+  else if (old_info != NULL)
+    {
+      gf_icon_set_file (GF_ICON (old_info->icon), new_file);
+    }
+  else if (new_info != NULL)
+    {
+      gf_icon_set_file (GF_ICON (new_info->icon), new_file);
+    }
 }
 
 static void
@@ -1598,6 +1630,8 @@ next_files_cb (GObject      *object,
       icon = gf_icon_new (self, file, info);
       g_object_unref (file);
 
+      g_signal_connect (icon, "changed", G_CALLBACK (icon_changed_cb), self);
+
       icon_info = create_icon_info (self, icon);
       self->icons = g_list_prepend (self->icons, icon_info);
     }
@@ -2023,6 +2057,17 @@ activate_cb (GfIconView *self,
     gf_icon_open (GF_ICON (l->data));
 }
 
+static void
+rename_cb (GfIconView *self,
+           gpointer    user_data)
+{
+  if (self->selected_icons == NULL ||
+      g_list_length (self->selected_icons) != 1)
+    return;
+
+  gf_icon_rename (GF_ICON (self->selected_icons->data));
+}
+
 static void
 trash_cb (GfIconView *self,
           gpointer    user_data)
@@ -2615,6 +2660,11 @@ install_signals (void)
                   G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
                   0, NULL, NULL, NULL, G_TYPE_NONE, 0);
 
+  view_signals[RENAME] =
+    g_signal_new ("rename", GF_TYPE_ICON_VIEW,
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                  0, NULL, NULL, NULL, G_TYPE_NONE, 0);
+
   view_signals[TRASH] =
     g_signal_new ("trash", GF_TYPE_ICON_VIEW,
                   G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
@@ -2687,6 +2737,12 @@ add_bindings (GtkBindingSet *binding_set)
   gtk_binding_entry_add_signal (binding_set, GDK_KEY_KP_Enter, modifiers,
                                 "activate", 0);
 
+  /* Rename */
+
+  modifiers = 0;
+  gtk_binding_entry_add_signal (binding_set, GDK_KEY_F2, modifiers,
+                                "rename", 0);
+
   /* Trash */
 
   modifiers = 0;
@@ -2782,6 +2838,7 @@ gf_icon_view_init (GfIconView *self)
   g_signal_connect (self, "select-all", G_CALLBACK (select_all_cb), NULL);
   g_signal_connect (self, "unselect-all", G_CALLBACK (unselect_all_cb), NULL);
   g_signal_connect (self, "activate", G_CALLBACK (activate_cb), NULL);
+  g_signal_connect (self, "rename", G_CALLBACK (rename_cb), NULL);
   g_signal_connect (self, "trash", G_CALLBACK (trash_cb), NULL);
   g_signal_connect (self, "delete", G_CALLBACK (delete_cb), NULL);
   g_signal_connect (self, "toggle", G_CALLBACK (toggle_cb), NULL);
