@@ -41,6 +41,8 @@ typedef struct
   GFile           *file;
   GFileInfo       *info;
 
+  GFileMonitor    *monitor;
+
   GfIconSize       icon_size;
   guint            extra_text_width;
 
@@ -83,6 +85,8 @@ enum
 {
   SELECTED,
 
+  CHANGED,
+
   LAST_SIGNAL
 };
 
@@ -280,7 +284,7 @@ drag_data_get_cb (GtkWidget        *widget,
 
       uris = get_selected_uris (self);
       gtk_selection_data_set_uris (data, uris);
-      g_free (uris);
+      g_strfreev (uris);
     }
 }
 
@@ -439,32 +443,7 @@ static void
 rename_cb (GtkMenuItem *item,
            GfIcon      *self)
 {
-  GfIconPrivate *priv;
-
-  priv = gf_icon_get_instance_private (self);
-
-  g_assert (priv->popover == NULL);
-  priv->popover = gf_rename_popover_new (GTK_WIDGET (self),
-                                         gf_icon_get_file_type (self),
-                                         gf_icon_get_name (self));
-
-  g_signal_connect (priv->popover, "validate",
-                    G_CALLBACK (rename_validate_cb),
-                    self);
-
-  g_signal_connect (priv->popover, "do-rename",
-                    G_CALLBACK (rename_do_rename_cb),
-                    self);
-
-  g_signal_connect (priv->popover, "closed",
-                    G_CALLBACK (rename_closed_cb),
-                    self);
-
-  g_signal_connect (priv->popover, "destroy",
-                    G_CALLBACK (rename_destroy_cb),
-                    self);
-
-  gtk_popover_popup (GTK_POPOVER (priv->popover));
+  gf_icon_rename (self);
 }
 
 static void
@@ -581,6 +560,9 @@ create_popup_menu (GfIcon *self)
       gtk_menu_shell_append (GTK_MENU_SHELL (popup_menu), item);
       gtk_widget_show (item);
 
+      label = gtk_bin_get_child (GTK_BIN (item));
+      gtk_accel_label_set_accel (GTK_ACCEL_LABEL (label), GDK_KEY_F2, 0);
+
       g_signal_connect (item, "activate",
                         G_CALLBACK (rename_cb),
                         self);
@@ -855,21 +837,33 @@ update_icon (GfIcon *self)
   gtk_image_set_pixel_size (GTK_IMAGE (priv->image), priv->icon_size);
 }
 
-static void
+static gboolean
 update_text (GfIcon *self)
 {
   GfIconPrivate *priv;
   const char *name;
+  char *old_name;
 
   priv = gf_icon_get_instance_private (self);
 
   name = GF_ICON_GET_CLASS (self)->get_text (self);
 
+  old_name = priv->name;
   priv->name = g_strdup (name);
+
   gtk_label_set_text (GTK_LABEL (priv->label), name);
 
   g_clear_pointer (&priv->name_collated, g_free);
   priv->name_collated = g_utf8_collate_key_for_filename (name, -1);
+
+  if (g_strcmp0 (old_name, name) != 0)
+    {
+      g_free (old_name);
+      return TRUE;
+    }
+
+  g_free (old_name);
+  return FALSE;
 }
 
 static void
@@ -944,7 +938,7 @@ load_thumbnail (GfIcon *self)
   g_free (uri);
 }
 
-static void
+static gboolean
 icon_refresh (GfIcon *self)
 {
   GfIconPrivate *priv;
@@ -968,7 +962,8 @@ icon_refresh (GfIcon *self)
     }
 
   update_icon (self);
-  update_text (self);
+
+  return update_text (self);
 }
 
 static void
@@ -1000,6 +995,8 @@ query_info_cb (GObject      *object,
   priv->info = file_info;
 
   icon_refresh (self);
+
+  g_signal_emit (self, icon_signals[CHANGED], 0);
 }
 
 static void
@@ -1026,6 +1023,70 @@ set_icon_size (GfIcon *self,
   update_icon (self);
 }
 
+static void
+file_changed_cb (GFileMonitor      *monitor,
+                 GFile             *file,
+                 GFile             *other_file,
+                 GFileMonitorEvent  event_type,
+                 GfIcon            *self)
+{
+  switch (event_type)
+    {
+      case G_FILE_MONITOR_EVENT_CHANGED:
+        break;
+
+      case G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
+        if (icon_refresh (self))
+          g_signal_emit (self, icon_signals[CHANGED], 0);
+        break;
+
+      case G_FILE_MONITOR_EVENT_DELETED:
+        break;
+
+      case G_FILE_MONITOR_EVENT_CREATED:
+        break;
+
+      case G_FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED:
+        gf_icon_update (self);
+        break;
+
+      case G_FILE_MONITOR_EVENT_PRE_UNMOUNT:
+        break;
+
+      case G_FILE_MONITOR_EVENT_UNMOUNTED:
+        break;
+
+      case G_FILE_MONITOR_EVENT_MOVED:
+        break;
+
+      case G_FILE_MONITOR_EVENT_RENAMED:
+        break;
+
+      case G_FILE_MONITOR_EVENT_MOVED_IN:
+        break;
+
+      case G_FILE_MONITOR_EVENT_MOVED_OUT:
+        break;
+
+      default:
+        break;
+    }
+}
+
+static void
+set_file (GfIcon *self,
+          GFile  *file)
+{
+  GfIconPrivate *priv;
+
+  priv = gf_icon_get_instance_private (self);
+
+  g_clear_object (&priv->file);
+  priv->file = g_object_ref (file);
+
+  GF_ICON_GET_CLASS (self)->create_file_monitor (self);
+}
+
 static void
 gf_icon_constructed (GObject *object)
 {
@@ -1053,6 +1114,8 @@ gf_icon_dispose (GObject *object)
   g_clear_object (&priv->file);
   g_clear_object (&priv->info);
 
+  g_clear_object (&priv->monitor);
+
   g_clear_object (&priv->app_info);
 
   g_clear_object (&priv->thumbnail);
@@ -1129,7 +1192,7 @@ gf_icon_set_property (GObject      *object,
 
       case PROP_FILE:
         g_assert (priv->file == NULL);
-        priv->file = g_value_dup_object (value);
+        set_file (self, g_value_get_object (value));
         break;
 
       case PROP_INFO:
@@ -1171,6 +1234,35 @@ gf_icon_get_preferred_width (GtkWidget *widget,
   *natural_width += priv->extra_text_width;
 }
 
+static void
+gf_icon_create_file_monitor (GfIcon *self)
+{
+  GfIconPrivate *priv;
+  GError *error;
+
+  priv = gf_icon_get_instance_private (self);
+
+  g_clear_object (&priv->monitor);
+
+  error = NULL;
+  priv->monitor = g_file_monitor_file (priv->file,
+                                       G_FILE_MONITOR_NONE,
+                                       NULL,
+                                       &error);
+
+  if (error != NULL)
+    {
+      g_warning ("%s", error->message);
+      g_error_free (error);
+      return;
+    }
+
+  g_signal_connect (priv->monitor,
+                    "changed",
+                    G_CALLBACK (file_changed_cb),
+                    self);
+}
+
 static GIcon *
 gf_icon_get_icon (GfIcon   *self,
                   gboolean *is_thumbnail)
@@ -1297,6 +1389,10 @@ install_signals (void)
   icon_signals[SELECTED] =
     g_signal_new ("selected", GF_TYPE_ICON, G_SIGNAL_RUN_LAST,
                   0, NULL, NULL, NULL, G_TYPE_NONE, 0);
+
+  icon_signals[CHANGED] =
+    g_signal_new ("changed", GF_TYPE_ICON, G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL, NULL, G_TYPE_NONE, 0);
 }
 
 static void
@@ -1316,6 +1412,7 @@ gf_icon_class_init (GfIconClass *self_class)
 
   widget_class->get_preferred_width = gf_icon_get_preferred_width;
 
+  self_class->create_file_monitor = gf_icon_create_file_monitor;
   self_class->get_icon = gf_icon_get_icon;
   self_class->get_text = gf_icon_get_text;
   self_class->can_delete = gf_icon_can_delete;
@@ -1433,9 +1530,7 @@ gf_icon_set_file (GfIcon *self,
 
   g_clear_pointer (&priv->popover, gtk_widget_destroy);
 
-  g_clear_object (&priv->file);
-  priv->file = g_object_ref (file);
-
+  set_file (self, file);
   gf_icon_update (self);
 }
 
@@ -1589,6 +1684,40 @@ gf_icon_open (GfIcon *self)
   g_free (uri);
 }
 
+void
+gf_icon_rename (GfIcon *self)
+{
+  GfIconPrivate *priv;
+
+  if (!GF_ICON_GET_CLASS (self)->can_rename (self))
+    return;
+
+  priv = gf_icon_get_instance_private (self);
+
+  g_assert (priv->popover == NULL);
+  priv->popover = gf_rename_popover_new (GTK_WIDGET (self),
+                                         gf_icon_get_file_type (self),
+                                         gf_icon_get_name (self));
+
+  g_signal_connect (priv->popover, "validate",
+                    G_CALLBACK (rename_validate_cb),
+                    self);
+
+  g_signal_connect (priv->popover, "do-rename",
+                    G_CALLBACK (rename_do_rename_cb),
+                    self);
+
+  g_signal_connect (priv->popover, "closed",
+                    G_CALLBACK (rename_closed_cb),
+                    self);
+
+  g_signal_connect (priv->popover, "destroy",
+                    G_CALLBACK (rename_destroy_cb),
+                    self);
+
+  gtk_popover_popup (GTK_POPOVER (priv->popover));
+}
+
 void
 gf_icon_popup_menu (GfIcon *self)
 {
