@@ -818,7 +818,13 @@ gf_unlock_dialog_dispose (GObject *object)
   g_clear_object (&self->dm_seat);
   g_clear_object (&self->login_seat);
 
-  g_clear_object (&self->auth);
+  if (self->auth != NULL)
+    {
+      gf_auth_cancel (self->auth);
+
+      g_object_unref (self->auth);
+      self->auth = NULL;
+    }
 
   G_OBJECT_CLASS (gf_unlock_dialog_parent_class)->dispose (object);
 }
@@ -1043,3 +1049,10 @@ gf_unlock_dialog_set_user_switch_enabled (GfUnlockDialog *self,
 
   update_user_switch_button (self);
 }
+
+void
+gf_unlock_dialog_forward_key_event (GfUnlockDialog *self,
+                                    GdkEvent       *event)
+{
+  gtk_widget_event (self->prompt_entry, event);
+}
